"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTwCall = void 0;
const core_1 = require("@babel/core");
const helpers_1 = require("./helpers");
function createTwCall(value, twFnName) {
    const twIdentifier = core_1.types.identifier(twFnName);
    if (core_1.types.isStringLiteral(value)) {
        return core_1.types.taggedTemplateExpression(twIdentifier, core_1.types.templateLiteral([core_1.types.templateElement({ raw: value.value, cooked: value.value }, true)], []));
    }
    if (core_1.types.isTemplateLiteral(value)) {
        return core_1.types.taggedTemplateExpression(twIdentifier, value);
    }
    if (core_1.types.isArrayExpression(value)) {
        // an element will be null in the case of holes in the array, e.g. [a,,c]
        // i believe we can just ignore those
        const elements = value.elements.filter(helpers_1.isTruthy);
        // for now, we won't bother trying to optimize this if there's a spread in the array
        // it's probably possible to do, but likely rare in practice
        if (onlyContainsExpressions(elements)) {
            const constantParts = [
                core_1.types.templateElement({ raw: '', cooked: '' }, true),
                ...Array.from({ length: elements.length - 1 }, () => core_1.types.templateElement({ raw: ' ', cooked: ' ' }, true)),
                core_1.types.templateElement({ raw: '', cooked: '' }, true),
            ];
            return core_1.types.taggedTemplateExpression(core_1.types.identifier(twFnName), core_1.types.templateLiteral(constantParts, elements));
        }
    }
    // for complex calls, pass the expression as-is
    return core_1.types.callExpression(core_1.types.identifier(twFnName), [value]);
}
exports.createTwCall = createTwCall;
function onlyContainsExpressions(nodes) {
    return nodes.every((node) => core_1.types.isExpression(node));
}
