"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.preprocess = exports.preprocessAst = void 0;
const tslib_1 = require("tslib");
/// <reference path="../types/react.d.ts" />
const babel = tslib_1.__importStar(require("@babel/core"));
const helper_annotate_as_pure_1 = tslib_1.__importDefault(require("@babel/helper-annotate-as-pure"));
const traverse_1 = tslib_1.__importDefault(require("@babel/traverse"));
const create_tw_call_1 = require("./create-tw-call");
const jsx_attribute_1 = require("./jsx-attribute");
/**
 * Mutatively(!) process an AST
 *
 * @param ast A babel AST
 */
function preprocessAst(ast) {
    traverse_1.default(ast, {
        Program(program) {
            const twFnName = program.scope.generateUid('tw');
            let hasTransformed = false;
            traverse_1.default(program.node, {
                JSXOpeningElement(path) {
                    const twAttribute = jsx_attribute_1.findJsxAttributeByName(path.node, 'tw');
                    const twAttributeValue = jsx_attribute_1.getJsxAttributeValue(twAttribute);
                    if (!twAttributeValue)
                        return;
                    const twCall = create_tw_call_1.createTwCall(twAttributeValue, twFnName);
                    helper_annotate_as_pure_1.default(twCall);
                    const classAttribute = jsx_attribute_1.findJsxAttributeByName(path.node, 'className');
                    const classAttributeValue = jsx_attribute_1.getJsxAttributeValue(classAttribute);
                    const newAttributeValue = classAttributeValue
                        ? babel.types.templateLiteral([
                            babel.types.templateElement({ raw: '' }),
                            babel.types.templateElement({ raw: ' ' }),
                            babel.types.templateElement({ raw: '' }),
                        ], [
                            // generate `${className || ''}`,
                            // so that we don't get `undefined` in the class string
                            // when the class name is undefined at runtime
                            //
                            // this can still be optimized though, e.g. in the case of a literal string,
                            // but I don't see that being common enough to account for
                            babel.types.logicalExpression('||', classAttributeValue, babel.types.stringLiteral('')),
                            twCall,
                        ])
                        : twCall;
                    path.node.attributes = path.node.attributes
                        // remove existing tw and className attributes
                        .filter((node) => {
                        const name = jsx_attribute_1.getJsxAttributeName(node);
                        return name !== 'tw' && name !== 'className';
                    })
                        // add new className attribute
                        .concat(babel.types.jsxAttribute(babel.types.jsxIdentifier('className'), babel.types.jsxExpressionContainer(newAttributeValue)));
                    hasTransformed = true;
                },
            });
            if (hasTransformed) {
                program.node.body.unshift(babel.types.importDeclaration([
                    babel.types.importSpecifier(babel.types.identifier(twFnName), babel.types.identifier('tw')),
                ], babel.types.stringLiteral('twind')));
            }
        },
    });
}
exports.preprocessAst = preprocessAst;
/**
 * Accept source code and process it
 */
async function preprocess(code, { filename } = {}) {
    const root = await babel.parseAsync(code, {
        configFile: false,
        babelrc: false,
        sourceMaps: true,
        parserOpts: {
            sourceFilename: filename,
        },
        plugins: [
            require.resolve('@babel/plugin-syntax-jsx'),
            [require.resolve('@babel/plugin-syntax-typescript'), { isTSX: true }],
        ],
    });
    if (!root)
        return undefined;
    preprocessAst(root);
    const result = await babel.transformFromAstAsync(root, code, {
        configFile: false,
        babelrc: false,
        ast: true,
        sourceMaps: true,
    });
    return result !== null && result !== void 0 ? result : undefined;
}
exports.preprocess = preprocess;
