"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getJsxAttributeValue = exports.getJsxAttributeName = exports.findJsxAttributeByName = void 0;
const core_1 = require("@babel/core");
function findJsxAttributeByName(jsxOpeningTag, name) {
    for (const node of jsxOpeningTag.attributes) {
        if (core_1.types.isJSXAttribute(node) && getJsxAttributeName(node) === name) {
            return node;
        }
    }
    return undefined;
}
exports.findJsxAttributeByName = findJsxAttributeByName;
function getJsxAttributeName(node) {
    if (core_1.types.isJSXAttribute(node) && core_1.types.isJSXIdentifier(node.name)) {
        return node.name.name;
    }
    return undefined;
}
exports.getJsxAttributeName = getJsxAttributeName;
function getJsxAttributeValue(attributeNode) {
    if (!core_1.types.isJSXAttribute(attributeNode))
        return;
    // not sure when this would happen 🤷‍♂️
    if (!attributeNode.value)
        return;
    // an expression container is like `attr={...}`,
    // so if it's not in a container,
    // we can just return the literal string or JSX node here
    if (!core_1.types.isJSXExpressionContainer(attributeNode.value)) {
        return attributeNode.value;
    }
    const innerExpression = attributeNode.value.expression;
    // this is the case of `attr={}`,
    // which is syntactically valid for some reason
    if (core_1.types.isJSXEmptyExpression(innerExpression))
        return undefined;
    return innerExpression;
}
exports.getJsxAttributeValue = getJsxAttributeValue;
